﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;   //必须添加，不然DllImport报错
using System.Windows.Forms;
using System.Threading;

namespace USB_6000_Ai_SoftTrig_Continues
{
    public partial class Form1 : Form
    {
        int DevIndex = 0;
        int ChanSel = 0;
        float Range = 10;
        double SampleRate;
        double MySampleRate;
        int SignalType = 0;

        TextBox[] Ai_Value = new TextBox[8];

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int temp;
            
            DevIndex = comboBox1.SelectedIndex;
            Range = float.Parse(comboBox2.Text);
            SampleRate = double.Parse(textBox1.Text);
            if (comboBox3.Text == "Ground")
            {
                SignalType = 3;
            }
            else if (comboBox3.Text == "DC Coupled")
            {
                SignalType = 0;
            }
            
            CheckBox[] Ai_checkBox = new CheckBox[8];
            Ai_checkBox[0] = checkBox1;
            Ai_checkBox[1] = checkBox2;
            Ai_checkBox[2] = checkBox3;
            Ai_checkBox[3] = checkBox4;
            Ai_checkBox[4] = checkBox5;
            Ai_checkBox[5] = checkBox6;
            Ai_checkBox[6] = checkBox7;
            Ai_checkBox[7] = checkBox8;
            
            int i;
            for (i = 0; i < 8; i++)
            {
                ChanSel = ChanSel << 1;
                ChanSel = ChanSel + (Ai_checkBox[7 - i].Checked ? 1 : 0);
            }

            double[] aa = new double[2];

            temp = USB6DLL.OpenDevice(DevIndex);                    // 打开设备
            temp = USB6DLL.SetClockSource(DevIndex, 1, 0);          // 设置时钟源为内部时钟源
            temp = USB6DLL.SetUSB6AiRange(DevIndex, 255, Range);    // 设置量程
            unsafe
            {
                fixed (double* a = &MySampleRate)
                {
                    temp = USB6DLL.SetSampleRate(DevIndex, SampleRate, a);      // 设置采样率
                }
            }
            temp = USB6DLL.CalibrateAi(DevIndex, 255);              // 执行自校准
            temp = USB6DLL.SetInputSignal(DevIndex, 255, SignalType);   // 设置输入通道信号类型
            
            Thread.Sleep(50);
            temp = USB6DLL.SetTrig(DevIndex, 0, 1, 0);              // 设置为软件触发
            temp = USB6DLL.SetChanSel(DevIndex, ChanSel);           // 设置通道选择
            temp = USB6DLL.StartRead(DevIndex);                     // 启动计算机读数线程
            Thread.Sleep(50);
            temp = USB6DLL.SetTrig(DevIndex, 0, 1, 1);              // 启动软件触发
            
            timer1.Enabled = true;
            button1.Enabled = false;
            button2.Enabled = true;
            button3.Enabled = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int temp;
            int i;
            temp = USB6DLL.FindUSBDAQ();            // 读取连接到计算机的采集卡数量
            if (temp > 0)
            {
                for (i = 0; i < temp; i++)
                {
                    comboBox1.Items.Add(i);
                }
                comboBox1.SelectedIndex = 0;
            }

            comboBox2.Items.Add(10);
            comboBox2.Items.Add(1);
            comboBox2.Items.Add(0.1);
            comboBox2.SelectedIndex = 0;

            textBox1.Text = "50000";

            comboBox3.Items.Add("Ground");
            comboBox3.Items.Add("DC Coupled");
            comboBox3.SelectedIndex = 0;

            Ai_Value[0] = textBox3;
            Ai_Value[1] = textBox4;
            Ai_Value[2] = textBox5;
            Ai_Value[3] = textBox6;
            Ai_Value[4] = textBox7;
            Ai_Value[5] = textBox8;
            Ai_Value[6] = textBox9;
            Ai_Value[7] = textBox10;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int temp;
            float[] Ai = new float[80000];

            // 从数据缓存fifo中读数
            temp = USB6DLL.GetAi(DevIndex, (uint)MySampleRate / 5, ChanSel, Ai, 1000);

            int i;
            int j;
            int k = 0;
            double data = 0;
            int mChanSel = ChanSel;
            for (i = 0; i < 8; i++)
            {
                if ((mChanSel & 1) > 0)
                {
                    for (j = 0; j < MySampleRate / 5; j++)
                    {
                        data = data + Ai[k * (int)(MySampleRate / 5) + j];
                    }
                    data = data / (MySampleRate / 5);
                    Ai_Value[i].Text = data.ToString();
                    data = 0;
                    k = k + 1;
                }
                else
                {
                    Ai_Value[i].Text = "0";
                }

                mChanSel = mChanSel >> 1;
            }

        }

        private void button2_Click(object sender, EventArgs e)
        {
            int temp;

            timer1.Enabled = false;
            Thread.Sleep(10);

            temp = USB6DLL.StopRead(DevIndex);                  // 停止计算机读数线程
            temp = USB6DLL.SetTrig(DevIndex, 0, 1, 0);          // 关闭软件触发
            temp = USB6DLL.ClearTrigger(DevIndex);              // 清空触发标志
            temp = USB6DLL.ClearDataFifo(DevIndex);             // 清空数据缓存fifo
            temp = USB6DLL.SetInputSignal(DevIndex, 255, 3);    // 将模拟输入通道接地
            USB6DLL.CloseDevice(DevIndex);                      // 关闭设备

            button1.Enabled = true;
            button2.Enabled = false;
            button3.Enabled = true;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.Close();
            Application.Exit();
        }
    }
    
    class USB6DLL
    {
        [DllImport("USB6.dll")]
        public static extern int FindUSBDAQ();
        [DllImport("USB6.dll")]
        public static extern int OpenDevice(int DevIndex);
        [DllImport("USB6.dll")]
        public static extern void CloseDevice(int DevIndex);

        [DllImport("USB6.dll")]
        public static extern int SetUSB6AiRange(int DevIndex, int Chan, float Range);
        [DllImport("USB6.dll")]
        unsafe public static extern int SetSampleRate(int DevIndex, double SampleRate, double* MySampleRate);
        [DllImport("USB6.dll")]
        public static extern int SetInputSignal(int DevIndex, int Chan, int SignalType);
        [DllImport("USB6.dll")]
        public static extern int SetClockSource(int DevIndex, int ClockSource, uint ExtClkFreq);
        [DllImport("USB6.dll")]
        public static extern int SetPfi(int DevIndex, int PfiMode, int Pfi_DO);
        [DllImport("USB6.dll")]
        public static extern int SetTrig(int DevIndex, int TrigSource, int TrigEdge, int SoftTrig);
        [DllImport("USB6.dll")]
        public static extern int ClearTrigger(int DevIndex);
        [DllImport("USB6.dll")]
        public static extern int SetChanSel(int DevIndex, int ChanSel);
        [DllImport("USB6.dll")]
        public static extern int ClearDataFifo(int DevIndex);

        [DllImport("USB6.dll")]
        public static extern int StartRead(int DevIndex);
        [DllImport("USB6.dll")]
        public static extern int StopRead(int DevIndex);

        [DllImport("USB6.dll")]
        public static extern int CalibrateAi(int DevIndex, int Chan);
        [DllImport("USB6.dll")]
        public static extern int SetCaliSource(int DevIndex, byte CaliSource);

        [DllImport("USB6.dll")]
        public static extern int GetAi(int DevIndex, uint Points, int ChSel, float[] Ai, long TimeOut);
    }
}
